<?php

/**
 * Ubercart Hotel Booking module -- Admin specific functions
 *
 * @file
 */

/**
 * Admin Settings page callback.
 */
function hotel_booking_admin_settings() {
  $form = array();
  global $base_url;

  $form['display'] = array(
    '#type'        => 'fieldset',
    '#title'       => t('General Settings'),
  );
  $form['display']['hotel_booking_display_nodeview'] = array(
    '#type'          => 'checkbox',
    '#title'         => t('Node view page'),
    '#description'   => t('If unchecked, calendars will not be displayed on node pages, but room availability also can not be edited. If you prefer to run with this off, you will need to turn it on to update availabilities.'),
    '#default_value' => variable_get('hotel_booking_display_nodeview', TRUE),
  );
  $form['display']['hotel_booking_display_monthcount'] = array(
    '#type'          => 'textfield',
    '#title'         => t('Number of months to display'),
    '#default_value' => variable_get('hotel_booking_display_monthcount', 3),
    '#description'   => t("Calendars will show this number of months to all users except those with the 'edit hotel room availability' permissions who will always see a full year (from current month forward)."),
  );
  $form['display']['hotel_booking_display_calprices'] = array(
    '#type'          => 'checkbox',
    '#title'         => t('Display prices in availability calendars'),
    '#description'   => t('If unchecked, prices will not be displayed in availability calendars.'),
    '#default_value' => variable_get('hotel_booking_display_calprices', TRUE),
  );
  $form['display']['redirect'] = array(
    '#type'          => 'fieldset',
    '#title'         => t('Browser Redirect Options'),
    '#collapsible'   => TRUE,
    '#collapsed'     => TRUE,
  );
  if (module_exists('hotel_addons_upgrades')) {
    $redirect_destination = array('cart' => t('Cart'), 'addon' => t('Upgrade/Addon form'), 'other' => t('Other'));
  } else
  {
    $redirect_destination = array('cart' => t('Cart'), 'other' => t('Other'));
  }
  $form['display']['redirect']['hotel_booking_redirect'] = array(
    '#type'          => 'radios',
    '#title'         => t('Where to redirect when a hotel room booking is added to the cart'),
    '#default_value' => variable_get('hotel_booking_redirect', 'cart'),
    '#options'       => $redirect_destination,
  );
  $form['display']['redirect']['hotel_booking_redirect_otherurl'] = array(
    '#type'          => 'textfield',
    '#title'         => t('URL to redirect to'),
    '#default_value' => variable_get('hotel_booking_redirect_otherurl', ''),
    '#description'   => t("If redirect destination of 'other' is selected above, enter the URL here, otherwise leave blank."),
    '#field_prefix'  => $base_url .'/',
  );
  if (module_exists('hotel_addons_upgrades')) {
    $form['display']['redirect']['addon'] = array(
      '#type'          => 'fieldset',
      '#title'         => t('Upgrade/Addon Page settings'),
      '#collapsible'   => TRUE,
      '#collapsed'     => TRUE,
    );
    $form['display']['redirect']['addon']['page_title'] = array(
      '#type'          => 'textfield',
      '#title'         => t('Upgrade/Addon Page Title'),
      '#default_value' => variable_get('hotel_booking_addons_page_title', ''),
    );
    $form['display']['redirect']['addon']['page_text'] = array(
      '#type'          => 'textarea',
      '#title'         => t('Upgrade/Addon Page Text'),
      '#default_value' => variable_get('hotel_booking_addons_page_text', '<p>The following items are available during your stay.</p>'),
      '#description'   => t('This text will appear at the top of the Upgrade/Addon page, before the list of available upgrade items. HTML tags are acceptable'),
    );
  }
  $form['block'] = array(
    '#type'          => 'fieldset',
    '#title'         => t('Block Settings'),
  );
  // Build list of room types
  $room_types[0] = t('None');
  $results = db_query('SELECT hrt.nid, n.title FROM {hotel_room_types} AS hrt, {node} AS n WHERE n.nid = hrt.nid');
  while ($result = db_fetch_array($results)) {
    $room_types['node/'. $result['nid']] = check_plain($result['title']);
  }
  $form['block']['best_rate_link'] = array(
    '#type'          => 'select',
    '#title'         => t('Room with best rate'),
    '#default_value' => variable_get('hotel_booking_best_rate', '0'),
    '#options'       => $room_types,
    '#description'   => t("Select the room type with the best rate to display a link to it in the search block, if no room type is selected, the link will not be displayed."),
  );
  $form['block']['search_children'] = array(
    '#type'          => 'select',
    '#title'         => t('Children Selection'),
    '#default_value' => variable_get('hotel_booking_block_children', '1'),
    '#options'       => array(0 => 'No', 1 => 'Yes'),
    '#description'   => t('Select whether or not to display the children selection option in search block. Value will be 0 for searches if No is selected here.'),
  );
  $form['block']['search_smoking'] = array(
    '#type'          => 'select',
    '#title'         => t('Smoking Preference Selection'),
    '#default_value' => variable_get('hotel_booking_block_smoking', '1'),
    '#options'       => array(0 => 'No', 1 => 'Yes'),
    '#description'   => t('Select whether or not to display the smoking preference selection option in search block. Value will be No Preference if No is selected here.'),
  );
  $form['search'] = array(
    '#type'        => 'fieldset',
    '#title'       => t('Search Result Options'),
  );
  $form['search']['sort'] = array(
    '#type'        => 'radios',
    '#title'       => t('Sort Order'),
    '#options'     => array('ASC' => t('Lowest price rooms first'), 'DESC' => t('Highest price rooms first')),
    '#default_value' => variable_get('hotel_booking_sort_order', 'ASC'),
    '#description' => t('The order to sort room search results.'),
  );
  $form['search']['search_results_teaser_body'] = array(
    '#type'        => 'radios',
    '#title'       => t('Room Description'),
    '#options'     => array('teaser' => t('Teaser'), 'body' => t('Body')),
    '#default_value' => variable_get('hotel_booking_search_teaser_body', 'teaser'),
    '#description' => t('Whether to display short teaser or full body text for room description.'),
  );

  $form['search']['no_search_results'] = array(
    '#type'        => 'textarea',
    '#title'       => t('Empty Search Result Message'),
    '#default_value' => variable_get('hotel_booking_no_search_results',  t('<p>Please select different dates and try your search again.</p>')),
    '#description' => t('This text will be displayed to users when a search returns no results. HTML is valid in this textarea.<br /><strong>Note:</strong> This will always be preceeded by <i>Your search returned 0 results.</i>'),
  );
  $form['expiry'] = array(
    '#type'        => 'fieldset',
    '#title'       => t('Expiry Settings'),
    '#description' => t('Select when you want available rooms to auto expire.'),
  );
  $form['expiry']['hotel_booking_expiration'] = array(
    '#type'        => 'radios',
    '#title'       => t('Close Availability'),
    '#options'     => array('yesterday' => t('Yesterday'), 'today' => t('Today'), 'tomorrow' => t('Tomorrow'), ),
    '#description' => t('Availability will be zeroed out by cron for dates <em>Equal To AND Older Than</em> this setting.'),
    '#default_value' => variable_get('hotel_booking_expiration', 'today'),
  );
  $form['cart'] = array(
    '#type'        => 'fieldset',
    '#title'       => t('Cart & Checkout settings'),
  );
  $form['cart']['teaser'] = array(
    '#type'        => 'radios',
    '#title'       => t('Teaser in cart'),
    '#options'     => array(1 => t('Yes'), 0 => t('No'), ),
    '#description' => t('Whether or not to include the teaser text in the cart. Applies to both Room Bookings and Upgrade/Addon Products'),
    '#default_value' => variable_get('hotel_booking_teaser_in_cart', 0),
  );
  $form['cart']['pane_teaser'] = array(
    '#type'        => 'radios',
    '#title'       => t('Teaser in cart, checkout, and order panes'),
    '#options'     => array(1 => t('Yes'), 0 => t('No'), ),
    '#description' => t('Whether or not to include the teaser text in the Booking Details panes on cart, checkout, and order review pages.'),
    '#default_value' => variable_get('hotel_booking_teaser_in_panes', 0),
  );
  $form['cart']['hotel_reward_card'] = array(
    '#type'          => 'textfield',
    '#title'         => t('Reward Membership Prompt'),
    '#default_value' => variable_get('hotel_booking_reward_prompt', ''),
    '#description'   => t('If you would like to prompt the user for a Reward/Membership number at checkout, enter the text to prompt them with here. Leave blank to disable the checkout field.'),
  );
  $form['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Save configuration'),
  );
  return $form;
}



function hotel_booking_admin_settings_submit($form, &$form_state) {
  variable_set('hotel_booking_display_nodeview', $form_state['values']['hotel_booking_display_nodeview']);
  variable_set('hotel_booking_display_calprices', $form_state['values']['hotel_booking_display_calprices']);
  variable_set('hotel_booking_display_monthcount', $form_state['values']['hotel_booking_display_monthcount']);
  variable_set('hotel_booking_expiration', $form_state['values']['hotel_booking_expiration']);
  variable_set('hotel_booking_redirect', $form_state['values']['hotel_booking_redirect']);
  variable_set('hotel_booking_redirect_otherurl', $form_state['values']['hotel_booking_redirect_otherurl']);
  variable_set('hotel_booking_sort_order', $form_state['values']['sort']);
  variable_set('hotel_booking_teaser_in_cart', $form_state['values']['teaser']);
  variable_set('hotel_booking_teaser_in_panes', $form_state['values']['pane_teaser']);
  variable_set('hotel_booking_reward_prompt', $form_state['values']['hotel_reward_card']);
  variable_set('hotel_booking_best_rate', $form_state['values']['best_rate_link']);
  variable_set('hotel_booking_addons_page_title', $form_state['values']['page_title']);
  variable_set('hotel_booking_addons_page_text', $form_state['values']['page_text']);
  variable_set('hotel_booking_no_search_results', $form_state['values']['no_search_results']);
  variable_set('hotel_booking_block_children', $form_state['values']['search_children']);
  variable_set('hotel_booking_block_smoking', $form_state['values']['search_smoking']);
  variable_set('hotel_booking_search_teaser_body', $form_state['values']['search_results_teaser_body']);
  drupal_set_message(t('Settings have been saved.'));
}



function hotel_booking_show_rates() {

  $hotel_rates = hotel_booking_rates_table();
  $hotel_rates .= l(t('Add Base Rate'), 'admin/store/hotel_booking/rates/add');

  $element = array(
    '#title' => t('Base Rates'),
    '#children' => $hotel_rates,
  );

  return theme('fieldset', $element);
}



function hotel_booking_rates_table() {
  $header = array(
    t('Rate Name'),
    array('data' => t('Actions'), 'colspan' => 2),
  );

  $rows = array();
  $rate_list = hotel_booking_get_rates();
  foreach ($rate_list as $rate) {
    $rows[] = array(
      l($rate->name, 'admin/store/hotel_booking/rates/'. $rate->hrid),
      l(t('Edit'), 'admin/store/hotel_booking/rates/'. $rate->hrid),
      l(t('Delete'), 'admin/store/hotel_booking/rates/'. $rate->hrid .'/delete'),
    );
  }
  $table = theme('table', $header, $rows);

  return $table;
}




function hotel_booking_show_rate_modifiers() {
  $hotel_rate_modifiers = hotel_booking_rate_modifiers_table();
  $hotel_rate_modifiers .= l(t('Add Rate Modifier'), 'admin/store/hotel_booking/rate_modifiers/add');

  $element = array(
    '#title' => t('Rate Modifiers'),
    '#children' => $hotel_rate_modifiers,
  );

  return theme('fieldset', $element);
}



function hotel_booking_rate_modifiers_table() {
  $header = array(
    t('Amount'),
    array('data' => t('Actions'), 'colspan' => 2),
  );

  $rows = array();
  $modifier_list = hotel_booking_get_rate_modifiers();
  foreach ($modifier_list as $modifier) {
    $amount = (check_plain($modifier->method) == 'percent') ? '&nbsp;&nbsp;'. number_format(check_plain($modifier->value), 2, '.', ',') .' %' : '$ '. number_format(check_plain($modifier->value), 2, '.', ',');
    $rows[] = array(
      $amount,
      l(t('Edit'), 'admin/store/hotel_booking/rate_modifiers/'. $modifier->hrmid),
      l(t('Delete'), 'admin/store/hotel_booking/rate_modifiers/'. $modifier->hrmid .'/delete'),
    );
  }
  $table = theme('table', $header, $rows);

  return $table;
}



function hotel_booking_show_occupancy_modifiers() {
  $hotel_rate_modifiers = hotel_booking_occupancy_modifiers_table();
  $hotel_rate_modifiers .= l(t('Add Occupancy Modifier'), 'admin/store/hotel_booking/occupancy_modifiers/add');

  $element = array(
    '#title' => t('Occupancy Modifiers'),
    '#children' => $hotel_rate_modifiers,
    '#description' => t('Amounts are added to room cost, for each night, for each additional guest exceeding the threshold.'),
  );

  return theme('fieldset', $element);
}



function hotel_booking_occupancy_modifiers_table() {
  $header = array(
    t('Threshold'),
    t('Amount'),
    array('data' => t('Actions'), 'colspan' => 2),
  );

  $rows = array();
  $modifier_list = hotel_booking_get_occupancy_modifiers();
  foreach ($modifier_list as $modifier) {
    if (check_plain($modifier->adult_child) == "adult") {
      $criteria = format_plural($modifier->threshold, '1 adult', '@count adults');
    }
    else {
      $criteria = format_plural($modifier->threshold, '1 child', '@count children');
    }
    $rows[] = array(
      $criteria,
      '$ '. number_format(check_plain($modifier->value), 2, '.', ','),
      l(t('Edit'), 'admin/store/hotel_booking/occupancy_modifiers/'. $modifier->hoid),
      l(t('Delete'), 'admin/store/hotel_booking/occupancy_modifiers/'. $modifier->hoid .'/delete'),
    );
  }
  $table = theme('table', $header, $rows);

  return $table;
}



/**
 * Hotel Booking Rate Modifiers form.
 */

function hotel_booking_rate_modifiers_form($form_state, $modifier_id = 0) {
    $modifier = hotel_booking_rate_modifiers_load_rate($modifier_id);

  if ($modifier->hrmid == 0) {
    drupal_set_title(t('New Rate Modifier'));
  }
  else {
    drupal_set_title(t('Rate Modifier #!modifier', array('!modifer' => $modifier->hrmid)));
  }

  $form = array();
  $form['hrmid'] = array(
    '#type'     => 'value',
    '#value'    => $modifier->hrmid,
  );

  $form['method'] = array(
    '#type'          => 'radios',
    '#title'         => t('Method'),
    '#description'   => 'Which method will be used to apply this modifier, dollar value or percentage of base rate',
    '#options'       => array('percent' => 'Percentage', 'value' => 'Dollar Value'),
    '#default_value' => isset($modifier->method) ? check_plain($modifier->method) : 'value',
  );
  $form['value'] = array(
    '#type'          => 'textfield',
    '#title'         => t('Adjustment Amount'),
    '#required'      => TRUE,
    '#default_value' => number_format($modifier->value, 2, '.', ','),
  );

  if ($modifier->hrmid == 0) {
    $form['submit'] = array(
      '#type' => 'submit',
      '#value' => t('Save'),
    );
  }
  else {
    $form['submit'] = array(
      '#type' => 'submit',
      '#value' => t('Update'),
    );
    $form['delete_button'] = array(
      '#type' => 'submit',
      '#name' => 'delete',
      '#value' => t('Delete'),
    );
  }
  return $form;
}




function hotel_booking_occupancy_modifiers_form($form_state, $hoid = 0) {
    $modifier = hotel_booking_occupancy_modifiers_load_rate($hoid);

  if ($modifier->hoid == 0) {
    drupal_set_title(t('New Occupancy Modifier'));
  }
  else {
    drupal_set_title(t('Occupancy Modifier #!modifier', array('!modifier' => $modifier->hoid)));
  }

  $form = array();
  $form['hoid'] = array(
    '#type'     => 'value',
    '#value'    => $modifier->hoid,
  );

  $form['adult_child'] = array(
    '#type'        => 'radios',
    '#title'       => t('Guest type'),
    '#description' => t('Select whether this modifier applies to adults, children, or overall number of occupants'),
    '#options'     => array('adult' => 'Adults', 'child' => 'Children', 'occup' => 'Occupants'),
    '#default_value' => isset($modifier->adult_child) ? check_plain($modifier->adult_child) : 'adult',
  );
  $form['threshold'] = array(
    '#title'       => t('Threshold'),
    '#type'        => 'textfield',
    '#required'    => TRUE,
    '#default_value' => $modifier->threshold,
    '#description' => t('Each guest over this threshold will be charged the additional adjustment amount per night.'),
  );
  $form['value'] = array(
    '#type'          => 'textfield',
    '#title'         => t('Adjustment Amount'),
    '#required'      => TRUE,
    '#field_prefix'  => '$ ',
    '#default_value' => number_format($modifier->value, 2, '.', ','),
  );

  if ($modifier->hoid == 0) {
    $form['submit'] = array(
      '#type' => 'submit',
      '#value' => t('Save'),
    );
  }
  else {
    $form['submit'] = array(
      '#type' => 'submit',
      '#value' => t('Update'),
    );
    $form['delete_button'] = array(
      '#type' => 'submit',
      '#name' => 'delete',
      '#value' => t('Delete'),
    );
  }
  return $form;
}



/**
 * Hotel Booking Rates form.
 */

function hotel_booking_rates_form($form_state, $rate_id = 0) {
    $rate = hotel_booking_rates_load_rate($rate_id);

  if ($rate->hrid == 0) {
    drupal_set_title(t('New Base Rate'));
  }
  else {
    drupal_set_title($rate->name);
  }

  $sign_flag = variable_get('uc_sign_after_amount', FALSE);
  $currency_sign = variable_get('uc_currency_sign', '$');

  $form['rate_id'] = array(
    '#type'          => 'value',
    '#value'         => $rate->hrid,
  );
  $form['name'] = array(
    '#title'         => t('Base Rate Name'),
    '#type'          => 'textfield',
    '#required'      => TRUE,
    '#default_value' => $rate->name,
  );
  $form['calendars'] = array(
    '#title'         => t('Rate Calendar'),
    '#type'          => 'fieldset',
    '#collapsible'   => FALSE,
    '#description'   => t('<p>Click a month to edit its rates.<br />Only dates with a value greater than zero will be saved to the database.</p><p>Prior months are removed from the database to save space.</p>'),
  );

  // Generate 12 months of calendars
  for ($i = 0; $i < 12; $i++) {
    $year = date("Y");
    $month = date("m") + $i;
    if ($month > 12) {
      $month -= 12;
      $year++;
    }

    $form['calendars'][$i] = array(
      '#title'       => t(date("F Y", mktime(0, 0, 0, $month, 1, $year))),
      '#type'        => 'fieldset',
      '#collapsible' => TRUE,
      '#collapsed'   => TRUE
    );

    $form['calendars'][$i]['rate_all_'. $i] = array(
        '#title'        => t('Set all '. date("F Y", mktime(0, 0, 0, $month, 1, $year)) .' rates to'),
        '#type'         => 'textfield',
        '#size'         => 12,
        '#attributes'   => array('class' => 'uc_hotel_set_all uc_hotel_rate'. $i)
    );
    $form['calendars'][$i]['set_all_'. $i] = array(
        '#value'        => t('Apply'),
        '#type'         => 'button',
        '#attributes'   => array('class'    => 'uc_hotel_set_all',
                                 'rel'      => 'uc_hotel_rate'. $i)
    );
    $form['calendars'][$i]['rates'] = array(
        '#type'         => 'markup',
        '#value'        => hotel_booking_rate_calendar($month, $year, $rate->hrid, $i));
    }

    $path = drupal_get_path('module', 'hotel_booking');
    drupal_add_js($path .'/js/hotel_booking.js', 'module', 'header');



  if ($rate->hrid == 0) {
    $form['submit'] = array(
      '#type' => 'submit',
      '#value' => t('Save'),
    );
  }
  else {
    $form['submit'] = array(
      '#type' => 'submit',
      '#value' => t('Update'),
    );
    $form['delete_button'] = array(
      '#type' => 'submit',
      '#name' => 'delete',
      '#value' => t('Delete'),
    );
  }
  return $form;
}



/**
 * Hotel Booking Rates form submission
 */
function hotel_booking_rates_form_submit($form, &$form_state) {

  if ($form_state['values']['rate_id'] == 0) {
    $sql = "INSERT INTO {hotel_rate_types} (name) VALUES ('%s')";
    db_query($sql, $form_state['values']['name']);
    $id = db_last_insert_id('{hotel_rate_types}', 'hrid');
  }
  else {
    $id = $form_state['values']['rate_id'];
    $sql = "UPDATE {hotel_rate_types} SET name='%s' WHERE hrid=%d";
    db_query($sql, $form_state['values']['name'], $id);
  }

  // First, remove all the old rate info...
  db_query('DELETE FROM {hotel_rate_calendars} WHERE hrid = %d', $id);

  foreach ($_POST['price'] as $key => $value) {
  // Loop through all of the price fields and save to the db any that are not 0.
    if (is_numeric($value) && ($value > 0)) {
      db_query('INSERT INTO {hotel_rate_calendars} (hrid, caldate, rate) VALUES (%d, "%s", %f)', $id, $key, $value);
    }
  }

  $form_state['redirect'] = "admin/store/hotel_booking/rates";
}


function hotel_booking_rate_modifiers_form_submit($form, &$form_state) {
  if ($form_state['values']['hrmid'] == 0) {
    $sql = "INSERT INTO {hotel_rate_modifiers} (method, value) VALUES ('%s', %f)";
    db_query($sql, $form_state['values']['method'], $form_state['values']['value']);
  }
  else {
    $sql = "UPDATE {hotel_rate_modifiers} SET method = '%s', value = %f WHERE hrmid = %d";
    db_query($sql, $form_state['values']['method'], $form_state['values']['value'], $form_state['values']['hrmid']);
  }

  $form_state['redirect'] = "admin/store/hotel_booking/rate_modifiers";
}



function hotel_booking_occupancy_modifiers_form_submit($form, &$form_state) {
  if ($form_state['values']['hoid'] == 0) {
    $sql = "INSERT INTO {hotel_occupancy_modifiers} (adult_child, threshold, value) VALUES ('%s', %d, %f)";
    db_query($sql, $form_state['values']['adult_child'], $form_state['values']['threshold'], $form_state['values']['value']);
  }
  else {
    $sql = "UPDATE {hotel_occupancy_modifiers} SET adult_child = '%s', threshold = %d, value = %f WHERE hoid = %d";
    db_query($sql, $form_state['values']['adult_child'], $form_state['values']['threshold'], $form_state['values']['value'], $form_state['values']['hoid']);
  }

  $form_state['redirect'] = "admin/store/hotel_booking/occupancy_modifiers";
}


/**
 * Hotel Booking Rates form validation
 */
function hotel_booking_rates_form_validate($form, &$form_state) {
  for ($i=0; $i<12; $i++) {
    if (isset($form_state['set_all_'. $i])) {
        if (isset($form_state['rate_all_'. $i])) {
            $year = date("Y");
            $month = date("m") + $i;
            if ($month > 12) {
              $month -= 12;
              $year++;
            }
            $days_in_month = _days_in_month($month, $year);
            $day_num = 1;

            while ($day_num <= $days_in_month) {
                $form_state['price['. $year .'-'. hotel_booking_pad_with_zeros($month, 2) .'-'. hotel_booking_pad_with_zeros($day_num, 2) .']'] = $form_state['rate_all_'. $i];
            }
        }
        else {
            form_set_error('calendars]['. $i .'][rate_all_'. $i, 'You must provide a rate to set all values to');
        }
    }
  }
  if (isset($form_state['values']['delete'])) {
    drupal_goto('admin/store/hotel_booking/rates/'. $form_state['values']['rate_id'] .'/delete');
  }
}



function hotel_booking_rate_modifiers_form_validate($form, &$form_state) {
  if (!is_numeric($form_state['values']['value'])) {
    form_set_error('value', t('You must enter a numeric value.'));
  }
  if (isset($form_state['values']['delete'])) {
    drupal_goto('admin/store/hotel_booking/rate_modifiers/'. $form_state['values']['hrmid'] .'/delete');
  }
}


function hotel_booking_occupancy_modifiers_form_validate($form, &$form_state) {
  if (!is_numeric($form_state['values']['value'])) {
    form_set_error('value', t('You must enter a numeric value.'));
  }
  if (isset($form_state['values']['delete'])) {
    drupal_goto('admin/store/hotel_booking/occupancy_modifiers/'. $form_state['values']['hoid'] .'/delete');
  }
}



function hotel_booking_rates_load_rate($rate_id = 0) {
  return db_fetch_object(db_query('SELECT * from {hotel_rate_types} WHERE hrid = %d', $rate_id));
}



function hotel_booking_rate_modifiers_load_rate($modifier_id = 0) {
  return db_fetch_object(db_query('SELECT * from {hotel_rate_modifiers} WHERE hrmid = %d', $modifier_id));
}


function hotel_booking_occupancy_modifiers_load_rate($hoid = 0) {
  return db_fetch_object(db_query('SELECT * from {hotel_occupancy_modifiers} WHERE hoid = %d', $hoid));
}


/**
 * Hotel Booking Rates deletion form.
 *
 * @param $discount_id
 *   Must be a valid and existing discount
 */
function hotel_booking_rates_delete_form($form_state, $rate_id = 0) {
  $rate = hotel_booking_rates_load_rate($rate_id);

  if ($rate == 0) {
    drupal_goto('admin/store/hotel_booking/rates');
  }
  if (!$rate) {
    drupal_goto('admin/store/hotel_booking/rates');
  }

  drupal_set_title(t('Delete !name', array('!name' => $rate->name)));

  $form['rate_id'] = array(
    '#type' => 'value',
    '#value' => $rate->hrid,
  );
  $form['rate_name'] = array(
    '#type' => 'value',
    '#value' => $rate->name,
  );
  $output = confirm_form($form,
    t('Delete !name?', array('!name' => $rate->name)),
    'admin/store/hotel_booking/rates/'. $rate->hrid,
    '',
    t('Delete'),
    t('Cancel')
  );

  return $output;
}



function hotel_booking_rate_modifiers_delete_form($form_state, $hrmid = 0) {
  $rate = hotel_booking_rate_modifiers_load_rate($hrmid);

  if ($rate == 0) {
    drupal_goto('admin/store/hotel_booking/rate_modifiers');
  }
  if (!$rate) {
    drupal_goto('admin/store/hotel_booking/rate_modifiers');
  }

  drupal_set_title(t('Delete Rate Modifier #!name', array('!name' => $rate->hrmid)));

  $form['hrmid'] = array(
    '#type' => 'value',
    '#value' => $rate->hrmid,
  );
  $output = confirm_form($form,
    t('Delete Modifier?'),
    'admin/store/hotel_booking/rate_modifiers/'. $rate->hrmid,
    '',
    t('Delete'),
    t('Cancel')
  );

  return $output;
}



function hotel_booking_occupancy_modifiers_delete_form($form_state, $hoid = 0) {
  $rate = hotel_booking_occupancy_modifiers_load_rate($hoid);
  if ($rate == 0) {
    drupal_goto('admin/store/hotel_booking/occupancy_modifiers');
  }
  if (!$rate) {
    drupal_goto('admin/store/hotel_booking/occupancy_modifiers');
  }

  drupal_set_title(t('Delete Occupancy Modifier #!name', array('!name' => $rate->hoid)));

  $form['hoid'] = array(
    '#type' => 'value',
    '#value' => $rate->hoid,
  );
  $output = confirm_form($form,
    t('Delete Modifier?'),
    'admin/store/hotel_booking/occupancy_modifiers/'. $rate->hoid,
    '',
    t('Delete'),
    t('Cancel')
  );

  return $output;
}


/**
 * Hotel Booking Rates deletion form submissions
 */
function hotel_booking_rates_delete_form_submit($form, $form_state) {
  if ($form_state['values']['confirm']) {
    db_query('DELETE FROM {hotel_rate_types} WHERE hrid=%d', $form_state['values']['rate_id']);
    db_query('DELETE FROM {hotel_rate_calendars} WHERE hrid = %d', $form_state['values']['rate_id']);

    drupal_set_message(t('Rate "!name" has been deleted', array('!name' => $form_state['values']['rate_name'])));
  }

  return 'admin/store/hotel_booking/rates';
}


function hotel_booking_rate_modifiers_delete_form_submit($form, $form_state) {
  if ($form_state['values']['confirm']) {
    db_query('DELETE FROM {hotel_rate_modifiers} WHERE hrmid=%d', $form_state['values']['hrmid']);
  }

  return 'admin/store/hotel_booking/rate_modifiers';
}


function hotel_booking_occupancy_modifiers_delete_form_submit($form, $form_state) {
  if ($form_state['values']['confirm']) {
    db_query('DELETE FROM {hotel_occupancy_modifiers} WHERE hoid = %d', $form_state['values']['hoid']);
    db_query('DELETE FROM {hotel_room_occupancy_modifiers} WHERE hoid = %d', $form_state['values']['hoid']);
  }

  return 'admin/store/hotel_booking/occupancy_modifiers';
}


/**
 * Base Rate Calendar
 */
function hotel_booking_rate_calendar($month = 1, $year = 2009, $hrid = 1, $group = 1) {

  $header = array(t('<center>Sunday</center>'), t('<center>Monday</center>'), t('<center>Tuesday</center>'), t('<center>Wednesday</center>'), t('<center>Thursday</center>'), t('<center>Friday</center>'), t('<center>Saturday</center>'));

  $result = db_query('SELECT rate, caldate FROM {hotel_rate_calendars} WHERE hrid = %d AND caldate BETWEEN "%s-%s-01" AND "%s-%s-%s"', $hrid, $year, hotel_booking_pad_with_zeros($month, 2), $year, hotel_booking_pad_with_zeros($month, 2), _days_in_month($month, $year));

  while ($results = db_fetch_array($result)) {
    // Extract DAY from the crappy datetime DB field type.
    $tmp = explode("-", $results['caldate']);
    $foo = explode(" ", $tmp[2]);
    $day = (int)$foo[0];

    $day_prices[$day]['rate'] = $results['rate'];
  }

  $rows = array();
  $days_in_month = _days_in_month($month, $year);

 // Generate the first day of the month
  $first_day = mktime(0, 0, 0, $month, 1, $year);

  // Set $title to the name of the month
  $title = date('F', $first_day);

  $day_of_week = date('D', $first_day);

  // Once we know what day of the week it falls on, we know how many blank days occur before it.  If the first day of the week is a Sunday then it would be zero.
  switch ($day_of_week) {
    case "Sun": $blank = 0; break;
    case "Mon": $blank = 1; break;
    case "Tue": $blank = 2; break;
    case "Wed": $blank = 3; break;
    case "Thu": $blank = 4; break;
    case "Fri": $blank = 5; break;
    case "Sat": $blank = 6; break;
  }

  $week = 1;
  $day_count = 1;

  while ($blank > 0) {
    $rows[($week - 1)][($day_count - 1)] .= '';
    $blank--;
    $day_count++;
  }

  $day_num = 1;

  while ($day_num <= $days_in_month) {
    $output = '<div style="text-align:center">'. $day_num .'</div>';
    $output .= '$ <input class="form-text uc_hotel_rate'. $group .'" type="text" size = "12" name="price['. $year .'-'. hotel_booking_pad_with_zeros($month, 2) .'-'. hotel_booking_pad_with_zeros($day_num, 2) .']" value="';
    $output .= (isset($day_prices[$day_num]['rate']) && is_numeric($day_prices[$day_num]['rate'])) ? number_format($day_prices[$day_num]['rate'], 2, '.', '') : '0.00';
    $output .= '" />';

    $rows[($week - 1)][($day_count - 1)] = $output;

    $day_num++;
    $day_count++;

    // Make sure we start a new row every week
    if ($day_count > 7) {
      $week++;
      $day_count = 1;
    }
  }

  // Finaly we finish out the calendar with some blanks if needed
  while ($day_count >1 && $day_count <=7) {
    $rows[($week - 1)][($day_count - 1)] = '';
    $day_count++;
  }

  $table = theme('table', $header, $rows);

  return $table;
}


function _days_in_month($month, $year) {
  // This array sets up the baseline of number of days in a month (starts with January).
  $md = array(31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31);

  if ($month < 1 || $month > 12) {
    return 0;
  }
  $d = $md[$month - 1];
  if ($month == 2) {
    // If it's a leap year, set Feb to 29 days.
    $d = ($d + date("L", mktime(0, 0, 0, $month, 1, $year)));
  }
  return $d;
}
